Package: BindingSiteFinder
Type: Package
Title: Binding site defintion based on iCLIP data
Version: 2.11.0
Authors@R: c(
    person("Mirko", "Brüggemann",
        email="mirko.brueggemann@mail.de",
        role = c("aut", "cre"),
        comment = c(ORCID = "0000-0002-1778-0248")),
    person("Melina", "Klostermann",
        email="melinaklostermann@googlemail.com",
        role = c("aut"),
        comment = c(ORCID = "0000-0003-3122-1095")),
    person("Kathi", "Zarnack",
        email="kathi.zarnack@bmls.de",
        role = c("aut"),
        comment = c(ORCID = "0000-0003-3527-3378")))
Description: Precise knowledge on the binding sites of an RNA-binding 
    protein (RBP) is key to understand (post-) transcriptional regulatory 
    processes. Here we present a workflow that describes how exact 
    binding sites can be defined from iCLIP data. The package provides functions
    for binding site definition and result visualization. For details please
    see the vignette. 
License: Artistic-2.0
Encoding: UTF-8
VignetteBuilder: knitr
Imports: tidyr, tibble, plyr, matrixStats, stats, ggplot2, methods,
        rtracklayer, S4Vectors, ggforce, GenomeInfoDb, ComplexHeatmap,
        RColorBrewer, lifecycle, rlang, forcats, dplyr,
        GenomicFeatures, IRanges, kableExtra, ggdist
Depends: GenomicRanges, R (>= 4.2)
Suggests: testthat, BiocStyle, knitr, rmarkdown, GenomicAlignments,
        scales, Gviz, xlsx, GGally, patchwork, viridis, ggplotify,
        SummarizedExperiment, DESeq2, ggpointdensity, ggrastr, ashr,
        txdbmaker, ggrepel, stringr
RoxygenNote: 7.3.2
Collate: 'AllClasses.R' 'AllGenerics.R' 'Functions.R' 'methods.R'
        'bindingsites.R' 'helper.R' 'PlotFunction.R'
        'CoverageFunctions.R' 'workflow.R' 'helperSpecific.R'
        'exports.R' 'coveragePlots.R' 'bsfind.R' 'helperPlots.R'
        'differentialFunctions.R' 'differentialPlots.R'
biocViews: Sequencing, GeneExpression, GeneRegulation,
        FunctionalGenomics, Coverage, DataImport
BugReports: https://github.com/ZarnackGroup/BindingSiteFinder/issues
git_url: https://git.bioconductor.org/packages/BindingSiteFinder
git_branch: devel
git_last_commit: aec001f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:30:59 UTC; biocbuild
Author: Mirko Brüggemann [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1778-0248>),
  Melina Klostermann [aut] (ORCID:
    <https://orcid.org/0000-0003-3122-1095>),
  Kathi Zarnack [aut] (ORCID: <https://orcid.org/0000-0003-3527-3378>)
Maintainer: Mirko Brüggemann <mirko.brueggemann@mail.de>
