# Generated by roxygen2: do not edit by hand

export(BSFDataSet)
export(BSFDataSetFromBigWig)
export(BSFind)
export(annotateWithScore)
export(assignToGenes)
export(assignToTranscriptRegions)
export(bindingSiteCoveragePlot)
export(bindingSiteDefinednessPlot)
export(calculateBsBackground)
export(calculateBsFoldChange)
export(calculateSignalToFlankScore)
export(clipCoverage)
export(collapseReplicates)
export(combineBSF)
export(coverageOverRanges)
export(duplicatedSitesPlot)
export(estimateBsWidth)
export(estimateBsWidthPlot)
export(exportTargetGenes)
export(exportToBED)
export(filterBsBackground)
export(geneOverlapsPlot)
export(geneRegulationPlot)
export(getMeta)
export(getName)
export(getRanges)
export(getSignal)
export(getSummary)
export(globalScorePlot)
export(imputeBsDifferencesForTestdata)
export(makeBindingSites)
export(makeBsSummaryPlot)
export(mergeCrosslinkDiagnosticsPlot)
export(mergeSummaryPlot)
export(plotBsBackgroundFilter)
export(plotBsMA)
export(plotBsVolcano)
export(processingStepsFlowChart)
export(processingStepsTable)
export(pureClipGeneWiseFilter)
export(pureClipGlobalFilter)
export(pureClipGlobalFilterPlot)
export(quickFigure)
export(rangeCoveragePlot)
export(reproducibilityCutoffPlot)
export(reproducibilityFilter)
export(reproducibilityFilterPlot)
export(reproducibilitySamplesPlot)
export(reproducibilityScatterPlot)
export(setMeta)
export(setName)
export(setRanges)
export(setSignal)
export(setSummary)
export(supportRatio)
export(supportRatioPlot)
export(targetGeneSpectrumPlot)
export(transcriptRegionOverlapsPlot)
export(transcriptRegionSpectrumPlot)
exportMethods("+")
exportMethods("[")
exportMethods(getMeta)
exportMethods(getName)
exportMethods(getRanges)
exportMethods(getSignal)
exportMethods(getSummary)
exportMethods(setMeta)
exportMethods(setName)
exportMethods(setRanges)
exportMethods(setSignal)
exportMethods(setSummary)
exportMethods(show)
exportMethods(summary)
import(ComplexHeatmap)
import(GenomicRanges)
import(ggplot2)
import(lifecycle)
import(methods)
import(rtracklayer)
import(tidyr)
importFrom(GenomeInfoDb,keepSeqlevels)
importFrom(GenomeInfoDb,seqlevels)
importFrom(GenomicFeatures,cds)
importFrom(GenomicFeatures,fiveUTRsByTranscript)
importFrom(GenomicFeatures,genes)
importFrom(GenomicFeatures,intronsByTranscript)
importFrom(GenomicFeatures,threeUTRsByTranscript)
importFrom(GenomicRanges,findOverlaps)
importFrom(IRanges,extractList)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,count)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,lead)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,reframe)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_tail)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,tally)
importFrom(dplyr,ungroup)
importFrom(ggdist,stat_halfeye)
importFrom(ggforce,geom_mark_rect)
importFrom(grDevices,colorRampPalette)
importFrom(grid,gpar)
importFrom(kableExtra,kable)
importFrom(matrixStats,colSds)
importFrom(plyr,rbind.fill)
importFrom(rtracklayer,import)
importFrom(stats,density)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(stats,sd)
importFrom(tibble,add_row)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,write.csv)
