# BindingSiteFinder

Precise knowledge on the binding sites of an RNA-binding protein (RBP) is key to understand (post-) transcriptional regulatory processes.

The `BindingSiteFinder` software package provides a full workflow for RBP binding site definition. The package provides function as well as rich visualizations and is well integrated in state-of-the-art and widely used Bioconductor classes, such as `GenomicRanges` and `SummarizedExperiments`. For details please see the [vignette](https://bioconductor.org/packages/release/bioc/html/BindingSiteFinder.html).

For latest changes and updates please have a look at the [devel-page](https://www.bioconductor.org/packages/devel/bioc/html/BindingSiteFinder.html).

![](https://github.com/ZarnackGroup/BindingSiteFinder/blob/devel/vignettes/images/bsf_promotion.png)

