#' @rdname getRanges
#' @export
setGeneric("getRanges", function(object) {
    standardGeneric("getRanges")
})

#' @rdname getMeta
#' @export
setGeneric("getMeta", function(object) {
    standardGeneric("getMeta")
})

#' @rdname setMeta
#' @export
setGeneric("setMeta", function(object, ...) {
    standardGeneric("setMeta")
})

#' @rdname getSignal
#' @export
setGeneric("getSignal", function(object) {
    standardGeneric("getSignal")
})

#' @rdname setRanges
#' @export
setGeneric("setRanges", function(object, ...) {
    standardGeneric("setRanges")
})

#' @rdname setSignal
#' @export
setGeneric("setSignal", function(object, ...) {
    standardGeneric("setSignal")
})

#' @rdname setSummary
#' @export
setGeneric("setSummary", function(object, ...) {
    standardGeneric("setSummary")
})

#' @rdname getSummary
#' @export
setGeneric("getSummary", function(object, ...) {
    standardGeneric("getSummary")
})

#' @rdname setName
#' @export
setGeneric("setName", function(object, ...) {
    standardGeneric("setName")
})

#' @rdname getName
#' @export
setGeneric("getName", function(object) {
    standardGeneric("getName")
})
